unit _links;

(*

  Unit _LINKS
  -----------

  freeware by Blackhole Entertainment
  --------



 unit to create Start/Programs/Links
 -----------------------------------

 with tnx to the creator of the german DELPHI FAQ where I got the code
 for the Win95 part: http://pics.webset.de/faq.htm

 Instructions:
  1. include unit _links to your uses-clause
  2. define a variable for either Win31 or 9x

 Win95-specific:
 calls:
   CreateFolder(<GroupName>);
        <Groupname> : full path of Group

   CreateLink(<filename>,<Shortcutname>,<Shortcuttitle>);
        <filename>      : file to that the shortcut is pointing
        <Shortcutname>  : filename of Shortcut (Ending with : .lnk)
        <Shortcuttitel> : Description of Shortcut

   SpecialDirectory(<ID>);
        <ID> : Identifier for Systemfolder, possible values :
                Autostart : CSIDL_Startup
                Startmenu : CSIDL_Startmenu
                Programs  : CSIDL_Programs
                Favorites : CSIDL_Favorites
                Desktop   : CSIDL_Desktopdirectory
                "Send to"-dir : CSIDL_Sendto

   examples:
      CreateFolder(SpecialDirectory(CSIDL_Programs)+'Neu');
        ->Erstellt die Programmgruppe "Neu" im Programs-Men.

      CreateLink('C:\Test\Programm.exe',SpecialDirectory(CSIDL_Startup)+'Programm.lnk','Programm');
        ->Erstellt einen Link it dem Titel "Programm" auf die Datei "C:\Test\Programm.exe"
          in der Autostart-Gruppe.



 Win31-specific:
  3. get a TDDEClientConv-object from the System-tab and put it on your
     form (or your Data-Module)
  4. set the ConnectMode-property to "ddeManual"
  5. call TWin31.init using this TDDEClientConv as parameter
  6. now you can create folders and links in the start-programs and
  7. call TWin31.clear at the end.

  how it works:
    if you look into the win32-api help under "DDE Services" (or similar) and
    program manager (or similar), you will see, that the program manager serves
    as server and you can connect to it as client and call progman what to do...






  contact : blackhole@iname.com

  no warranty for anything! this is freeware, use it on your own risk!    

*)

interface

uses Windows, SysUtils, Classes, DdeMan,

{$IFDEF VER90}
 OLE2, ComObj, ShlObj;
{$ELSE}
 ActiveX, ComObj, ShlObj;
{$ENDIF}

type
 TWin95Links = object
   function CreateLink(lpszPathObj,lpszPathLink,lpszDesc:string):Boolean;
   function SpecialDirectory(ID:integer):string;
   function CreateFolder(Foldername:string):boolean;
 end;

 TWin31Links = object
   DDEClient : TDDEClientConv;
   procedure init (var DDE : TDDEClientConv);
   procedure clear;
   procedure CreateStartGroup (name : String);
   procedure CreateStartItem (name, Destination : String);
 end;



implementation



const
  IID_IPersistFile : TGUID=(D1:$0000010B;D2:$0000;D3:$0000;D4:($C0,$00,$00,$00,$00,$00,$00,$46));

function TWin95Links.SpecialDirectory(ID:integer):string;
var pidl : PItemIDList;
    Path : PChar;
begin
  if SUCCEEDED(SHGetSpecialFolderLocation(0,ID,pidl)) then begin
    Path:=StrAlloc(max_path);
    SHGetPathFromIDList(pidl,Path);
    Result:=String(Path);
    if Result[length(Result)]<>'\' then
      Result:=Result+'\';
  end;
end; {SpecialDirectory}

Function TWin95Links.CreateFolder(Foldername:string):boolean;
begin
  Result:=false;
  SetLastError(0);
  CreateDirectory(PChar(Foldername), nil );
  if (GetLastError()=0) or (GetLastError()=ERROR_ALREADY_EXISTS) then
    Result:=true;
end; {CreateFolder}

function TWin95Links.CreateLink(lpszPathObj,lpszPathLink,lpszDesc:string):Boolean;
var psl : IShellLink;
    ppf : IPersistFile;
begin
  Result:=false;
  if SUCCEEDED(CoCreateInstance(CLSID_ShellLink,nil,CLSCTX_INPROC_SERVER, IID_IShellLinkA, psl)) then begin
    psl.SetPath(PChar(lpszPathObj));
    psl.SetDescription(PChar(lpszDesc));
    if SUCCEEDED(psl.QueryInterface(IID_IPersistFile,ppf)) then begin
      ppf.Save(StringToOLEStr(lpszPathLink),TRUE);
      Result:=true;
    end;
  end;
end; {CreateLink}

{
Aufrufe:
CreateFolder(<GruppenName>);
        <Gruppenname> : Komplette Pfadangabe der Programmgruppe

CreateLink(<Dateiname>,<Shortcutname>,<Shortcuttitel>);
        <Dateiname> : Datei, auf die der Shortcut verweisen soll
        <Shortcutname> : Dateiname des Shortcuts (Endung : .lnk)
        <Shortcuttitel> : Beschreibung des Shortcuts

SpecialDirectory(<ID>);
        <ID> : Identifier fr Systemordner, mgliche Werte:
                Autostart-Gruppe : CSIDL_Startup
                Startmen : CSIDL_Startmenu
                Programs-Men : CSIDL_Programs
                Persnliche Favoriten : CSIDL_Favorites
                Desktop : CSIDL_Desktopdirectory
                "Send an"-Verzeichnis : CSIDL_Sendto

Beispiele :
CreateFolder(SpecialDirectory(CSIDL_Programs)+'Neu');
        ->Erstellt die Programmgruppe "Neu" im Programs-Men.

CreateLink('C:\Test\Programm.exe',SpecialDirectory(CSIDL_Startup)+'Programm.lnk','Programm');
        ->Erstellt einen Link it dem Titel "Programm" auf die Datei "C:\Test\Programm.exe"
          in der Autostart-Gruppe.
}


 procedure TWin31Links.init (var DDE : TDDEClientConv);
 begin
      DDEClient := DDE;
      DDEClient.SetLink('Progman','Progman');
      DDEClient.OpenLink;
 end;

 procedure TWin31Links.clear;
 begin
      DDEClient.CloseLink;
 end;

 procedure TWin31Links.CreateStartGroup (name : String);
  var S : TStringList;
 begin
     S := TStringList.Create;
     S.Add ('[CreateGroup ("'+name+'")]');
     DDEClient.ExecuteMacroLines (S, false);
 end;

 procedure TWin31Links.CreateStartItem (name, Destination : String);
  var S : TStringList;
      dummy : String;
 begin
     S := TStringList.Create;
     if Destination[length(Destination)]<>'\' then Destination := Destination+'\';
     dummy := '[AddItem ("'+Destination+'","'+name+'")]';
     S.Add (dummy);
//     S.Add ('[Reload ("Ethicon")]');
     DDEClient.ExecuteMacroLines (S, false);
 end;

end.
